@@ -4,11 +4,11 @@
 @Description: 算法提供者核心服务
 @Author: Kermit
 @Date: 2022-11-05 16:46:46
-@LastEditors: Kermit
-@LastEditTime: 2023-01-27 18:22:01
 '''
 
-from typing import Callable, Optional, List, Tuple
+from typing import Any, Callable, Optional, List, Tuple, Union
+from algospace.logger import Logger, algospace_logger
+from algospace.util import create_timestamp_filename
 from . import config
 from .config import Algoinfo
 import urllib.request
@@ -24,14 +24,17 @@ import traceback
 import requests
 import websocket
 from algospace.login import login, login_instance
-from .config_loader import ConfigLoader, valid_param_type
+from .config_loader import ConfigLoader, InputType, OutputType, valid_input_type, valid_output_type
+from .param_validator import ParamValidator
 from .enroll import enroll, verify_config, is_component_normal
 from .stdio import GradioPrint, QueueStdIO, QueueStdIOExec
-import shutil
+from algospace.exceptions import InvalidCallParamException
 import json
 import time
+import datetime
 import os
 import re
+import base64
 
 
 class FnService:
@@ -39,6 +42,8 @@ class FnService:
 
     def __init__(self, algorithm_config: ConfigLoader) -> None:
         self.algorithm_config = algorithm_config
+        self.param_validator = ParamValidator(algorithm_config)
+        self.logger = Logger('Fn Service', is_show_time=True)
 
     async def handle(self, *args, **kwargs):
         ''' 处理函数请求 '''
@@ -51,11 +56,12 @@ class FnService:
         while True:
             try:
                 args, kwargs = fn_req_queue.get()
-                print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                      f'[Fn Service] [{fn_index}] Begin to calculate')
+                self.logger.info(f'[{fn_index}] Begin to calculate.')
+                param_result, param_err_msg = self.param_validator.validate(*args, **kwargs)
+                if not param_result:
+                    raise InvalidCallParamException(param_err_msg)
                 out = asyncio.run(self.handle(*args, **kwargs))
-                print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                      f'[Fn Service] [{fn_index}] Complete.')
+                self.logger.info(f'[{fn_index}] Complete.')
                 fn_res_queue.put((out, None))
             except Exception as e:
                 fn_res_queue.put((None, e))
@@ -90,24 +96,41 @@ class ApiService:
     def __init__(self, algorithm_config: ConfigLoader, algorithm_info: Algoinfo) -> None:
         self.algorithm_config = algorithm_config
         self.algorithm_info = algorithm_info
+        self.logger = Logger('Api Service', is_show_time=True)
+
+    def read_file(self, dict_or_path: Union[dict, str]) -> str:
+        ''' 将文件字典或 storage 返回的 path 转换为本地文件并返回本地文件路径 '''
+        if type(dict_or_path) == dict:
+            file_dict: dict = dict_or_path  # type: ignore
+            file_name = create_timestamp_filename() + '.' + \
+                file_dict.get('format', '') if file_dict.get('format') else create_timestamp_filename()
+
+            if file_dict.get('type', 'base64') == 'base64':
+                file_bytes = base64.b64decode(file_dict.get('data', ''), validate=True)
+            elif file_dict.get('type', 'base64') == 'url':
+                file_bytes = requests.get(file_dict.get('data', '')).content
+            else:
+                raise Exception(f'Invalid file type: {file_dict.get("type")}, only support base64 and url.')
+        else:
+            # 兼容旧版本: 传入的是 storage 返回的 path
+            path: str = dict_or_path  # type: ignore
+            file_name = path.split('/')[-1]
+            file_url = config.storage_file_url + '/' + path
+            file_request = urllib.request.Request(url=file_url,  method='GET')
+            headers = login_instance.get_header()
+            for key, value in headers.items():
+                file_request.add_header(key, value)
+            file_response = urllib.request.urlopen(file_request)
+            file_bytes = file_response.read()
 
-    def read_file(self, path: str) -> str:
-        ''' 将 storage 返回的 path 转换为本地 path '''
-        file_url = config.storage_file_url + '/' + path
         tmp_path = self.algorithm_config.service_tmp_path
-        dir_path = os.path.dirname(os.path.join(tmp_path, path))
+        local_path = os.path.join(tmp_path, 'api', file_name)
+        dir_path = os.path.dirname(local_path)
         if not os.path.exists(dir_path):
             os.makedirs(dir_path)
 
-        file_request = urllib.request.Request(url=file_url,  method='GET')
-        headers = login_instance.get_header()
-        for key, value in headers.items():
-            file_request.add_header(key, value)
-        file_bytes = urllib.request.urlopen(file_request).read()
-
-        with open(os.path.join(tmp_path, path), 'wb') as f:
+        with open(local_path, 'wb') as f:
             f.write(file_bytes)
-        local_path = os.path.join(tmp_path, path)
 
         return local_path
 
@@ -129,40 +152,40 @@ class ApiService:
 
         return path
 
-    def get_input_type_class(self, type: str, ) -> Callable:
+    def get_input_type_class(self, input_type: str) -> Callable:
         ''' 获取处理每种类型的 Callable 对象 '''
-        if type not in valid_param_type:
-            raise Exception(f'Param type \'{type}\' is not available.')
-        elif type == 'str':
+        if input_type not in valid_input_type:
+            raise Exception(f'Input type \'{input_type}\' is not available.')
+        elif input_type == InputType.STRING:
             return str
-        elif type == 'int':
+        elif input_type == InputType.INTEGER:
             return int
-        elif type == 'float':
+        elif input_type == InputType.FLOAT:
             return float
-        elif type == 'image_path':
+        elif input_type == InputType.IMAGE_PATH:
             return self.read_file
-        elif type == 'video_path':
+        elif input_type == InputType.VIDEO_PATH:
             return self.read_file
-        elif type == 'voice_path':
+        elif input_type == InputType.VOICE_PATH:
             return self.read_file
         else:
             return str
 
-    def get_output_type_class(self, type: str, ) -> Callable:
+    def get_output_type_class(self, output_type: str) -> Callable:
         ''' 获取处理每种类型的 Callable 对象 '''
-        if type not in valid_param_type:
-            raise Exception(f'Param type \'{type}\' is not available.')
-        elif type == 'str':
+        if output_type not in valid_output_type:
+            raise Exception(f'Output type \'{output_type}\' is not available.')
+        elif output_type == OutputType.STRING:
             return str
-        elif type == 'int':
+        elif output_type == OutputType.INTEGER:
             return int
-        elif type == 'float':
+        elif output_type == OutputType.FLOAT:
             return float
-        elif type == 'image_path':
+        elif output_type == OutputType.IMAGE_PATH:
             return self.write_file
-        elif type == 'video_path':
+        elif output_type == OutputType.VIDEO_PATH:
             return self.write_file
-        elif type == 'voice_path':
+        elif output_type == OutputType.VOICE_PATH:
             return self.write_file
         else:
             return str
@@ -173,7 +196,7 @@ class ApiService:
                fn_req_queue_list: List[multiprocessing.Queue],
                fn_res_queue_list: List[multiprocessing.Queue]) -> dict:
         ''' 处理请求 '''
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[Api Service] Begin to handle.')
+        self.logger.info('Begin to handle.')
         params = {}
         for key, info in self.algorithm_config.service_input.items():
             params[key] = self.get_input_type_class(info['type'])(input_info[key])
@@ -193,7 +216,7 @@ class ApiService:
         output_info = {}
         for key, info in self.algorithm_config.service_output.items():
             output_info[key] = self.get_output_type_class(info['type'])(out[key])
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[Api Service] Complete.')
+        self.logger.info('Complete.')
         return output_info
 
     def get_example(self):
@@ -214,22 +237,43 @@ class GradioService:
         self.algorithm_config = algorithm_config
         self.algorithm_info = algorithm_info
         self.gradio_upload_url = None
+        self.logger = Logger('Gradio Service', is_show_time=True)
+        self.init_logger = Logger('Gradio Init', is_show_time=True)
 
-    def get_type_component(self, type: str, describe: str):
+    def get_input_component(self, input_type: str, describe: str, default: Any):
         ''' 获取处理每种类型的 Gradio component '''
-        if type not in valid_param_type:
-            raise Exception(f'Param type \'{type}\' is not available.')
-        elif type == 'str':
+        if input_type not in valid_input_type:
+            raise Exception(f'Input type \'{input_type}\' is not available.')
+        elif input_type == InputType.STRING:
+            return gr.Textbox(placeholder=describe, label=describe, value=default)
+        elif input_type == InputType.INTEGER:
+            return gr.Number(precision=0, label=describe, value=default)
+        elif input_type == InputType.FLOAT:
+            return gr.Number(label=describe, value=default)
+        elif input_type == InputType.IMAGE_PATH:
+            return gr.Image(type='filepath', label=describe, value=default)
+        elif input_type == InputType.VIDEO_PATH:
+            return gr.Video(type='filepath', label=describe, value=default)
+        elif input_type == InputType.VOICE_PATH:
+            return gr.Audio(type='filepath', label=describe, value=default)
+        else:
+            return gr.Textbox(placeholder=describe, value=default)
+
+    def get_output_component(self, output_type: str, describe: str):
+        ''' 获取处理每种类型的 Gradio component '''
+        if output_type not in valid_output_type:
+            raise Exception(f'Output type \'{output_type}\' is not available.')
+        elif output_type == OutputType.STRING:
             return gr.Textbox(placeholder=describe, label=describe)
-        elif type == 'int':
+        elif output_type == OutputType.INTEGER:
             return gr.Number(precision=0, label=describe)
-        elif type == 'float':
+        elif output_type == OutputType.FLOAT:
             return gr.Number(label=describe)
-        elif type == 'image_path':
+        elif output_type == OutputType.IMAGE_PATH:
             return gr.Image(type='filepath', label=describe)
-        elif type == 'video_path':
+        elif output_type == OutputType.VIDEO_PATH:
             return gr.Video(type='filepath', label=describe)
-        elif type == 'voice_path':
+        elif output_type == OutputType.VOICE_PATH:
             return gr.Audio(type='filepath', label=describe)
         else:
             return gr.Textbox(placeholder=describe)
@@ -264,17 +308,26 @@ class GradioService:
 
         inputs = []
         for _, info in self.algorithm_config.service_input.items():
-            inputs.append(self.get_type_component(info['type'], info['describe']))
+            inputs.append(self.get_input_component(info['type'], info['describe'], info.get('default', None)))
         outputs = []
         for _, info in self.algorithm_config.service_output.items():
-            outputs.append(self.get_type_component(info['type'], info['describe']))
+            outputs.append(self.get_output_component(info['type'], info['describe']))
+        examples = []
+        for input_example in self.algorithm_config.service_input_examples:
+            example = []
+            for key, _ in self.algorithm_config.service_input.items():
+                example.append(input_example[key])
+            examples.append(example)
 
         gr_interface = gr.Interface(
             title=self.algorithm_info.full_name,
-            # description=self.algorithm_config.description,
             fn=fn,
             inputs=inputs,
             outputs=outputs,
+            **({
+                'examples': examples,
+                'cache_examples': True
+            } if len(examples) > 0 else {})
         )
 
         self.check_port()
@@ -285,12 +338,13 @@ class GradioService:
                 show_api=False,
                 show_tips=False,
                 favicon_path=None,
+                show_error=True,
                 inline=True,
                 quiet=True,
                 height=500,
                 width=900,
                 server_name=self.algorithm_config.gradio_server_host,
-                server_port=self.algorithm_config.gradio_server_port
+                server_port=self.algorithm_config.gradio_server_port,
             )
 
     def check_port(self):
@@ -328,7 +382,7 @@ class GradioService:
 
     async def handle(self, input_info: dict) -> dict:
         ''' 处理请求 '''
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[Gradio Service] Begin to handle.')
+        self.logger.info('Begin to handle.')
 
         req_method = input_info['method']
         req_apipath = input_info['apipath']
@@ -360,53 +414,11 @@ class GradioService:
         else:
             result = {}
 
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[Gradio Service] Complete.')
+        self.logger.info('Complete.')
         return result
 
-    async def handle_init(self) -> dict:
-        ''' 处理 Gradio 初始化请求 '''
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[Gradio Init] Begin to handle.')
-
-        if not os.path.exists('./frontend'):
-            gradio_path = '/'.join(gr.__file__.split('/')[:-1])
-            gradio_frontend_path = os.path.join(gradio_path, 'templates/frontend')
-            shutil.copytree(gradio_frontend_path, './frontend')
-            os.remove('./frontend/index.html')  # 删除未渲染的 index.html
-
-        async def upload_file(bashpath: str, subpath: str):
-            ''' 递归上传文件夹里的文件 '''
-            path = os.path.join(bashpath, subpath)
-            if os.path.exists(os.path.join(path, '.DS_Store')):
-                os.remove(os.path.join(path, '.DS_Store'))
-            for filename in os.listdir(path):
-                filepath = os.path.join(path, filename)
-                if os.path.isdir(filepath):
-                    await upload_file(bashpath, os.path.join(subpath, filename))
-                else:
-                    url = self.gradio_upload_url + '/' + subpath + '/' + filename  # type: ignore
-                    file = open(filepath, 'r', encoding="UTF-8", errors='ignore')
-                    res = await asyncio.get_event_loop().run_in_executor(None, lambda: requests.post(url, files={'file': file}, headers=login_instance.get_header()))
-                    file.close()
-
-        await upload_file('./frontend', '.')  # 上传文件夹里的文件
-
-        if not os.path.exists('./frontend/index.html'):
-            html_res = await asyncio.get_event_loop().run_in_executor(None, lambda: requests.get(
-                f'http://{self.algorithm_config.gradio_server_host}:{self.algorithm_config.gradio_server_port}'))
-            html_text = html_res.text
-            html_file = open('./frontend/index.html', 'w+', encoding="UTF-8", errors='ignore')
-            html_file.write(html_text)
-            html_file.close()
-        url = self.gradio_upload_url + '/index.html'  # type: ignore
-        file = open('./frontend/index.html', 'r', encoding="UTF-8", errors='ignore')
-        # 上传 index.html
-        res = await asyncio.get_event_loop().run_in_executor(None, lambda: requests.post(url, files={'file': file}, headers=login_instance.get_header()))
-        file.close()
-
-        shutil.rmtree('./frontend')  # 删除已上传的文件夹
-
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[Gradio Init] Complete.')
-        return {}
+    def is_nouse_fn(self, input_info):
+        return type(input_info) == dict and input_info['method'] == 'GET'
 
 
 class Service:
@@ -428,6 +440,9 @@ class Service:
         self.api_service = ApiService(self.algorithm_config, self.algorithm_info)
         self.gradio_service = GradioService(self.algorithm_config, self.algorithm_info)
 
+        self.logger = Logger('Service', is_show_time=True)
+        self.algo_logger = Logger(self.algorithm_info.upper_name, is_show_time=True)
+
     def launch(self,
                stdio_queue: multiprocessing.Queue,
                type: str,
@@ -457,8 +472,7 @@ class Service:
                 raise Exception('Argument "type" is unaccessable.')
         except Exception as e:
             traceback.print_exc()
-            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                  f'[{self.algorithm_info.upper_name}] Launch {type} error:', str(e))
+            self.algo_logger.error(f'Launch {type} error: {str(e)}')
             exit(1)
 
     def launch_service(self,
@@ -472,9 +486,9 @@ class Service:
         self.algorithm_config.gradio_server_port = gradio_port
 
         # 开始处理
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}] [Service] Running in {self.fetch_mode} mode.')
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}] [Service] Your algorithm site: {self.algorithm_info.algorithm_site}')
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}] [Service] Start handling...')
+        self.logger.info(f'Running in {self.fetch_mode} mode.')
+        self.logger.info(f'Your algorithm site: {self.algorithm_info.algorithm_site}')
+        self.logger.info('Start handling...')
         print('')
 
         parallel = {
@@ -504,15 +518,15 @@ class Service:
             def on_req(req_info: dict, loop: asyncio.AbstractEventLoop):
                 # 处理请求
                 try:
-                    add_parallel(req_info['type'] == 'gradio_init')
+                    if not self.is_exclude_parallel(req_info):
+                        add_parallel()
                     future = asyncio.run_coroutine_threadsafe(self.handle(req_info, fn_lock_list,
                                                                           fn_req_queue_list, fn_res_queue_list), loop)
-                    future.add_done_callback(lambda task: minus_parallel(
-                        req_info['type'] == 'gradio_init'))  # type: ignore
+                    if not self.is_exclude_parallel(req_info):
+                        future.add_done_callback(lambda task: minus_parallel())  # type: ignore
                 except Exception as e:
                     traceback.print_exc()
-                    print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                          '[Service] Create handle task error:', str(e))
+                    self.logger.error(f'Create handle task error: {str(e)}')
 
                 while is_parallel_full():
                     # 并行数达到上限，等待
@@ -540,23 +554,22 @@ class Service:
                             continue
                     except Exception as e:
                         traceback.print_exc()
-                        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                              '[Service] Ask data error:', str(e))
+                        self.logger.error(f'Ask data error: {str(e)}')
                         await asyncio.sleep(config.wait_interval)
                         continue
 
                     # 处理请求
                     try:
-                        add_parallel(req_info['type'] == 'gradio_init')
+                        if not self.is_exclude_parallel(req_info):
+                            add_parallel()
                         future = loop.create_task(self.handle(req_info, fn_lock_list,
                                                               fn_req_queue_list, fn_res_queue_list))
-                        future.add_done_callback(lambda task: minus_parallel(
-                            req_info['type'] == 'gradio_init'))  # type: ignore
+                        if not self.is_exclude_parallel(req_info):
+                            future.add_done_callback(lambda task: minus_parallel())  # type: ignore
                         await asyncio.sleep(config.call_interval)
                     except Exception as e:
                         traceback.print_exc()
-                        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                              '[Service] Create handle task error:', str(e))
+                        self.logger.error(f'Create handle task error: {str(e)}')
                         await asyncio.sleep(config.wait_interval)
 
             asyncio.run(start())
@@ -594,8 +607,7 @@ class Service:
             await loop.run_in_executor(None, ws.run_forever)
         except Exception as e:
             traceback.print_exc()
-            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                  '[Service] Websocket ask data error:', str(e))
+            self.logger.error(f'Websocket ask data error: {str(e)}')
 
     async def ask_data(self, id: Optional[str] = None) -> Optional[dict]:
         ask_for_data_resp = await asyncio.get_event_loop().run_in_executor(None, lambda: requests.get(self.ask_data_url,
@@ -621,56 +633,60 @@ class Service:
             raise Exception('the property of \'req_info\' is not dict.')
         return req_info
 
+    def is_exclude_parallel(self, req_info: dict):
+        ''' 是否排除计算并行数外 '''
+        return req_info['type'] == 'gradio' and self.gradio_service.is_nouse_fn(req_info['data'])
+
+    async def get_full_req_info(self, req_info: dict):
+        ''' 获取完整的请求信息 '''
+        if type(req_info['data']) != dict:
+            req_id = req_info['id']
+            full_req_info = None
+            retry_times = 0
+            max_retry_times = 5
+            while not full_req_info:
+                if retry_times >= max_retry_times:
+                    raise Exception(f'Algo req \'{req_id}\' is not found.')
+                try:
+                    full_req_info = await self.ask_data(req_id)
+                except:
+                    pass
+                finally:
+                    retry_times += 1
+                    await asyncio.sleep(config.wait_interval * retry_times)
+            req_info = full_req_info
+        return req_info
+
     async def handle(self,
                      req_info: dict,
                      fn_lock_list: List[Lock],
                      fn_req_queue_list: List[multiprocessing.Queue],
                      fn_res_queue_list: List[multiprocessing.Queue]):
         try:
+            start_time = datetime.datetime.now()
             try:
-                if type(req_info['data']) != dict:
-                    req_id = req_info['id']
-                    full_req_info = None
-                    retry_times = 0
-                    max_retry_times = 5
-                    while not full_req_info:
-                        if retry_times >= max_retry_times:
-                            raise Exception(f'Algo req \'{req_id}\' is not found.')
-                        try:
-                            full_req_info = await self.ask_data(req_id)
-                        except:
-                            pass
-                        finally:
-                            retry_times += 1
-                            await asyncio.sleep(config.wait_interval * retry_times)
-                    req_info = full_req_info
+                req_info = await self.get_full_req_info(req_info)
 
                 # 根据请求类型处理
+                start_time = datetime.datetime.now()
                 if req_info['type'] == 'api':
                     # 处理计算请求
                     result = await asyncio.get_event_loop().run_in_executor(None, lambda: self.api_service.handle(req_info['data'], fn_lock_list, fn_req_queue_list, fn_res_queue_list))
                 elif req_info['type'] == 'gradio':
                     # 处理 Gradio 请求
                     result = await self.gradio_service.handle(req_info['data'])
-                elif req_info['type'] == 'gradio_init':
-                    # 处理 Gradio 初始化请求
-                    result = await self.gradio_service.handle_init()
                 else:
                     result = {}
+                end_time = datetime.datetime.now()
+                calculate_time = (end_time - start_time).total_seconds()
 
                 # 回传数据
-                res_info = {
-                    'id': req_info['id'],
-                    'name': req_info['name'],
-                    'version': req_info['version'],
-                    'type': req_info['type'],
-                    'status': 'success',
-                    'owner': req_info['owner'],
-                    'create_date': req_info['create_date'],
-                    'result': result
-                }
                 return_ans_param = {
-                    'res_info': res_info,
+                    'res_info': {
+                        'id': req_info['id'],
+                        'calculate_time': calculate_time,
+                        'result': result
+                    },
                     'algorithm_name': req_info['name'],
                     'algorithm_version': req_info['version']
                 }
@@ -682,39 +698,32 @@ class Service:
                     err_msg = "Service Result Return Error."
                     raise Exception(err_msg)
                 else:
-                    print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                          '[Service] Return ans success.')
+                    self.logger.info(f'Return ans success.')
             except Exception as e:
                 traceback.print_exc()
+                end_time = datetime.datetime.now()
+                calculate_time = (end_time - start_time).total_seconds()
                 # 回传失败数据
-                res_info = {
-                    'id': req_info['id'],
-                    'name': req_info['name'],
-                    'version': req_info['version'],
-                    'type': req_info['type'],
-                    'status': 'error',
-                    'owner': req_info['owner'],
-                    'create_date': req_info['create_date'],
-                    'result': {'err_msg': str(e)}
-                }
                 return_error_param = {
-                    'res_info': res_info,
+                    'res_info': {
+                        'id': req_info['id'],
+                        'calculate_time': calculate_time,
+                        'result': {'err_msg': str(e)}
+                    },
                     'algorithm_name': req_info['name'],
                     'algorithm_version': req_info['version']
                 }
                 await asyncio.get_event_loop().run_in_executor(None, lambda: requests.post(self.return_err_url,
                                                                                            json=return_error_param,
                                                                                            headers=login_instance.get_header()))
-                print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                      '[Service] Return error ans success.')
+                self.logger.error(f'Return error ans success.')
         except Exception as e:
             traceback.print_exc()
-            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[Service] Handle error:', str(e))
+            self.logger.error(f'Handle error: {str(e)}')
 
     def login(self):
-        if not login(self.algorithm_config.username, self.algorithm_config.password):
-            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                  f'[{self.algorithm_info.upper_name}] Login failed. Please check your password.')
+        if not login(secret=self.algorithm_config.secret, username=self.algorithm_config.username, password=self.algorithm_config.password, privilege='PROVIDER'):
+            self.algo_logger.error(f'Login failed. Please check your secret or password.')
             return False
         return True
 
@@ -731,8 +740,7 @@ class Service:
                    self.algorithm_config.config_file_content)
             return True
         except Exception as e:
-            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                  f'[{self.algorithm_info.upper_name}] Enroll failed:', str(e))
+            self.algo_logger.error(f'Enroll failed: {str(e)}')
             return False
 
     def verify_config(self):
@@ -746,13 +754,12 @@ class Service:
                 filepath = os.path.join(dirpath, 'algospace-config-origin.py')
                 with open(filepath, 'w') as f:
                     f.write(file)
-                print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                      f'[{self.algorithm_info.upper_name}] Content of \'{self.algorithm_config.config_path}\' is not same as the config enrolled before. The original config file is regenerated at \'{filepath}\'. Please use the original file to start. Or if you have modified the algorithm, use a new \'version\' to start.')
+                self.algo_logger.error(
+                    f'Content of \'{self.algorithm_config.config_path}\' is not same as the config enrolled before. The original config file is regenerated at \'{filepath}\'. Please use the original file to start. Or if you have modified the algorithm, use a new \'version\' to start.')
                 return False
             return True
         except Exception as e:
-            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                  f'[{self.algorithm_info.upper_name}] Verify config failed:', str(e))
+            self.algo_logger.error(f'Verify config failed: {str(e)}')
             return False
 
     def is_component_normal(self):
@@ -765,8 +772,7 @@ class Service:
                 self.return_err_url = res['return_err_url']
                 self.gradio_service.gradio_upload_url = res['gradio_upload_url']
                 break
-            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                  f'[{self.algorithm_info.upper_name}] Waiting for components started...')
+            self.algo_logger.info(f'Waiting for components started...')
             time.sleep(5)
 
     def send_heartbeat(self):
@@ -778,8 +784,7 @@ class Service:
 
     async def start(self):
         # 初始化状态
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-              f'[{self.algorithm_info.upper_name}] Initializing...')
+        self.algo_logger.info(f'Initializing...')
         if not self.login():
             return
         if not self.enroll():
@@ -833,32 +838,23 @@ class Service:
             gradio_process.start()
             return gradio_process
 
-        def check_process_alive(process: multiprocessing.Process):
-            is_alive = process.is_alive()
-            if not is_alive:
-                process.kill()
-            return is_alive
-
         fn_process = create_fn_process()
         service_process = create_service_process()
         gradio_process = create_gradio_process()
+        process_exit_code = None
 
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-              f'[{self.algorithm_info.upper_name}] Waiting for service launched...')
+        self.algo_logger.info(f'Waiting for service launched...')
 
-        # 开启事件循环
-        async def alive_task():
-            nonlocal fn_process
-            nonlocal service_process
-            nonlocal gradio_process
-            while True:
-                await asyncio.sleep(1)
-                if not check_process_alive(fn_process):
-                    raise Exception('Fn process is not alive. Please check error log above.')
-                if not check_process_alive(service_process):
-                    raise Exception('Service process is not alive')
-                if not check_process_alive(gradio_process):
-                    raise Exception('Gradio process is not alive')
+        after_cancel_tasks = []
+
+        async def join_task(process: multiprocessing.Process):
+            nonlocal process_exit_code
+            try:
+                await asyncio.get_event_loop().run_in_executor(None, process.join)
+                process_exit_code = process.exitcode
+            except:
+                if process.is_alive():
+                    process.kill()
 
         async def heartbeat_task():
             times = 0
@@ -867,48 +863,65 @@ class Service:
                     await asyncio.sleep(1)
                     if times % 10 == 0:
                         # 每 10 秒发送心跳
-                        await asyncio.get_running_loop().run_in_executor(None, self.send_heartbeat)
+                        await asyncio.get_event_loop().run_in_executor(None, self.send_heartbeat)
                         times = 0
                     times += 1
+                except concurrent.futures._base.CancelledError:
+                    break
+                except asyncio.CancelledError:
+                    break
                 except Exception as e:
+                    times = 1
                     traceback.print_exc()
-                    print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                          f'[{self.algorithm_info.upper_name}] Heartbeat error:', str(e))
+                    self.algo_logger.error(f'Heartbeat error: {str(e)}')
 
         async def subprocess_stdio_task():
             queue_stdio_exec = QueueStdIOExec(stdio_queue)
             stdio_exec = queue_stdio_exec.exec
             stdio_exec_all = queue_stdio_exec.exec_all
+            after_cancel_tasks.append(stdio_exec_all)
             is_execed = True
             while True:
                 try:
                     if not is_execed:
                         await asyncio.sleep(0.1)
-                    is_execed = await asyncio.get_running_loop().run_in_executor(None, stdio_exec)
+                    is_execed = await asyncio.get_event_loop().run_in_executor(None, stdio_exec)
+                except concurrent.futures._base.CancelledError:
+                    break
+                except asyncio.CancelledError:
+                    break
                 except Exception as e:
                     traceback.print_exc()
-                    print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                          f'[{self.algorithm_info.upper_name}] Handle subprocess stdio error:', str(e))
-                except asyncio.CancelledError:
-                    stdio_exec_all()
-                    raise
+                    self.algo_logger.error(f'Handle subprocess stdio error: {str(e)}')
 
         # 当有任务抛出异常时，停止所有任务
-        tasks = [alive_task(), heartbeat_task(), subprocess_stdio_task()]
-        done, pending = await asyncio.wait(tasks, return_when=asyncio.FIRST_EXCEPTION)
+        tasks = [join_task(fn_process),
+                 join_task(service_process),
+                 join_task(gradio_process),
+                 heartbeat_task(),
+                 subprocess_stdio_task()]
+        done, pending = await asyncio.wait(tasks, return_when=asyncio.FIRST_COMPLETED)
         for task in pending:
             task.cancel()
+        for task in after_cancel_tasks:
+            task()
         for task in done:
             task.result()
+        return process_exit_code
 
 
 def run_service(config_path: str, fetch_mode: str = 'listen') -> None:
     loop = asyncio.get_event_loop()
+    service_coroutine = None
+    exit_code = None
     try:
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[AlgoSpace] Init.')
+        algospace_logger.info('Init.')
         service = Service(config_path, fetch_mode)
-        loop.run_until_complete(service.start())
+        service_coroutine = service.start()
+        exit_code = loop.run_until_complete(service_coroutine)
     except:
         traceback.print_exc()
+        exit_code = 1
     finally:
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[AlgoSpace] Exit.')
+        algospace_logger.info('Exit.')
+    exit(exit_code or 0)
